'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _envfile = require('envfile');

var _envfile2 = _interopRequireDefault(_envfile);

var _flowRuntime = require('flow-runtime');

var _flowRuntime2 = _interopRequireDefault(_flowRuntime);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getHardwareInfo = () => {
  let cpuInfo;
  try {
    cpuInfo = (0, _fs.readFileSync)('/proc/cpuinfo', { encoding: 'utf8' });
  } catch (e) {
    // if this fails, this is probably not a pi
    return {};
  }

  const data = new Map(cpuInfo.split('\n').map(line => line.replace(/\t/g, '')).filter(line => line.length).map(line => {
    let _lineType = _flowRuntime2.default.string();

    _flowRuntime2.default.param('line', _lineType).assert(line);

    const splitted = line.split(':');
    return [splitted[0].trim(), splitted.slice(1).join(':').trim()];
  }));

  return {
    model: data.get('Hardware'),
    serial: data.get('Serial'),
    revision: data.get('Revision')
  };
};

const getOsInfo = () => {
  const snakeCaseToCamelCase = s => {
    let _sType = _flowRuntime2.default.string();

    _flowRuntime2.default.param('s', _sType).assert(s);

    return s.toLowerCase().replace(/(_\w)/g, m => m[1].toUpperCase());
  };

  try {
    const parsed = _envfile2.default.parseFileSync('/etc/os-release');
    const osInfo = {};
    Object.keys(parsed).forEach(key => {
      osInfo[snakeCaseToCamelCase(key)] = parsed[key];
    });

    return osInfo;
  } catch (err) {
    console.warn(err);
    return {
      id: 'unknown'
    };
  }
};

const PI_MODEL_NO = ['BCM2708', 'BCM2709', 'BCM2710'];
const isPi = model => PI_MODEL_NO.includes(model);

const rpiVersion = () => {
  try {
    // eslint-disable-next-line
    return require('raspi-ver');
  } catch (err) {
    return null;
  }
};

const detect = () => {
  const hardwareInfo = getHardwareInfo();
  const osInfo = getOsInfo();

  const model = hardwareInfo.model;

  let user;

  if (osInfo.id === 'osmc') {
    user = 'osmc';
  }

  if (isPi(model)) {
    return {
      id: 'rpi',
      name: 'Raspberry PI',
      user: user || 'pi',
      info: rpiVersion(),
      os: osInfo,
      hardware: hardwareInfo
    };
  }

  if (model && model.startsWith('Rockchip')) {
    return {
      id: 'tkb',
      name: 'Tinker Board',
      user: 'linaro',
      os: osInfo,
      hardware: hardwareInfo
    };
  }

  if (model === 'Vero4K') {
    return {
      id: 'vero4k',
      name: 'Vero4K',
      user,
      os: osInfo,
      hardware: hardwareInfo
    };
  }

  return {
    id: 'unknown',
    name: 'Unknown',
    os: osInfo,
    hardware: hardwareInfo
  };
};

let cache;

exports.default = function detectBoard() {
  if (!cache) cache = detect();
  return cache;
};
//# sourceMappingURL=detectBoard.js.map